==> vungle-ads-7.4.0-sources.jar: com/vungle/ads/internal/load/RealtimeAdLoader.kt <==
package com.vungle.ads.internal.load

import android.content.Context
import com.vungle.ads.AdMarkupInvalidError
import com.vungle.ads.AnalyticsClient
import com.vungle.ads.ServiceLocator
import com.vungle.ads.VungleError
import com.vungle.ads.internal.ConfigManager
import com.vungle.ads.internal.downloader.Downloader
import com.vungle.ads.internal.executor.Executors
import com.vungle.ads.internal.network.TpatSender
import com.vungle.ads.internal.network.VungleApiClient
import com.vungle.ads.internal.omsdk.OMInjector
import com.vungle.ads.internal.signals.SignalManager
import com.vungle.ads.internal.util.Logger
import com.vungle.ads.internal.util.PathProvider

class RealtimeAdLoader(
    context: Context,
    vungleApiClient: VungleApiClient,
    sdkExecutors: Executors,
    omInjector: OMInjector,
    downloader: Downloader,
    pathProvider: PathProvider,
    adRequest: AdRequest
) : BaseAdLoader(
    context, vungleApiClient, sdkExecutors, omInjector, downloader, pathProvider, adRequest
) {

    override fun requestAd() {
        val adMarkup = adRequest.adMarkup
        // Support V2 only.
        if (adMarkup == null) {
            AnalyticsClient.logError(
                VungleError.INVALID_BID_PAYLOAD,
                "Unable to create data object from payload string.",
                adRequest.placement.referenceId
            )
            onAdLoadFailed(AdMarkupInvalidError())
            return
        }

        if (ConfigManager.rtaDebuggingEnabled()) {
            try {
                val adResponse = adMarkup.getDecodedAdsResponse()
                Logger.d("RTA_DEBUGGER", "$adResponse")
                val vungleApiClient: VungleApiClient by ServiceLocator.inject(context)
                adResponse?.let { RTADebugger(vungleApiClient).reportAdMarkup(it) }
            } catch (ignored: Throwable) {
                // ignore
            }
        }

        val adPayload = adMarkup.getAdPayload()

        if (adMarkup.version != 2 || adPayload == null) {
            AnalyticsClient.logError(
                VungleError.INVALID_ADUNIT_BID_PAYLOAD,
                "The ad response did not contain valid ad markup.",
                adRequest.placement.referenceId,
                eventId = adMarkup.getEventId()
            )
            onAdLoadFailed(AdMarkupInvalidError())
            return
        }

        handleAdMetaData(adPayload)
    }

    override fun onAdLoadReady() {
        sendWinNotification(advertisement?.getWinNotifications())
    }

    private fun sendWinNotification(notifications: List<String>?) {
        if (notifications?.isEmpty() == true) {
            return
        }
        val signalManager: SignalManager by ServiceLocator.inject(context)
        val tpatSender = TpatSender(
            vungleApiClient,
            adRequest.placement.referenceId,
            advertisement?.getCreativeId(),
            advertisement?.eventId(),
            sdkExecutors.ioExecutor,
            pathProvider,
            signalManager,
        )
        notifications?.forEach { notificationUrl ->
            tpatSender.sendWinNotification(notificationUrl, sdkExecutors.jobExecutor)
        }
    }
}

==> vungle-ads-7.4.1-sources.jar: com/vungle/ads/internal/load/RealtimeAdLoader.kt <==
package com.vungle.ads.internal.load

import android.content.Context
import com.vungle.ads.AdMarkupInvalidError
import com.vungle.ads.AnalyticsClient
import com.vungle.ads.ServiceLocator
import com.vungle.ads.SingleValueMetric
import com.vungle.ads.VungleError
import com.vungle.ads.internal.ConfigManager
import com.vungle.ads.internal.downloader.Downloader
import com.vungle.ads.internal.executor.Executors
import com.vungle.ads.internal.network.TpatSender
import com.vungle.ads.internal.network.VungleApiClient
import com.vungle.ads.internal.omsdk.OMInjector
import com.vungle.ads.internal.protos.Sdk
import com.vungle.ads.internal.signals.SignalManager
import com.vungle.ads.internal.util.Logger
import com.vungle.ads.internal.util.PathProvider

class RealtimeAdLoader(
    context: Context,
    vungleApiClient: VungleApiClient,
    sdkExecutors: Executors,
    omInjector: OMInjector,
    downloader: Downloader,
    pathProvider: PathProvider,
    adRequest: AdRequest
) : BaseAdLoader(
    context, vungleApiClient, sdkExecutors, omInjector, downloader, pathProvider, adRequest
) {

    override fun requestAd() {
        val adMarkup = adRequest.adMarkup
        // Support V2 only.
        if (adMarkup == null) {
            AnalyticsClient.logError(
                VungleError.INVALID_BID_PAYLOAD,
                "Unable to create data object from payload string.",
                adRequest.placement.referenceId
            )
            onAdLoadFailed(AdMarkupInvalidError())
            return
        }

        if (ConfigManager.rtaDebuggingEnabled()) {
            try {
                val adResponse = adMarkup.getDecodedAdsResponse()
                Logger.d("RTA_DEBUGGER", "$adResponse")
                val vungleApiClient: VungleApiClient by ServiceLocator.inject(context)
                adResponse?.let { RTADebugger(vungleApiClient).reportAdMarkup(it) }
            } catch (ignored: Throwable) {
                // ignore
            }
        }

        val adPayload = adMarkup.getAdPayload()

        if (adMarkup.version != 2 || adPayload == null) {
            AnalyticsClient.logError(
                VungleError.INVALID_ADUNIT_BID_PAYLOAD,
                "The ad response did not contain valid ad markup.",
                adRequest.placement.referenceId,
                eventId = adMarkup.getEventId()
            )
            onAdLoadFailed(AdMarkupInvalidError())
            return
        }

        handleAdMetaData(adPayload, SingleValueMetric(Sdk.SDKMetric.SDKMetricType.CONFIG_LOADED_FROM_ADM_LOAD))
    }

    override fun onAdLoadReady() {
        sendWinNotification(advertisement?.getWinNotifications())
    }

    private fun sendWinNotification(notifications: List<String>?) {
        if (notifications?.isEmpty() == true) {
            return
        }
        val signalManager: SignalManager by ServiceLocator.inject(context)
        val tpatSender = TpatSender(
            vungleApiClient,
            adRequest.placement.referenceId,
            advertisement?.getCreativeId(),
            advertisement?.eventId(),
            sdkExecutors.ioExecutor,
            pathProvider,
            signalManager,
        )
        notifications?.forEach { notificationUrl ->
            tpatSender.sendWinNotification(notificationUrl, sdkExecutors.jobExecutor)
        }
    }
}
